<?php

class Manifest_Builder {
	protected $_cache_files = array();
	protected $_network_files = array();
	protected $_fallbacks = array();
	protected $_manifest_path = '';
	protected $_version_hash = '';
	protected $_manifest_changed = TRUE;
	
	/**
	 * pushes a file to the given array
	 * @param array as reference
	 * @param string
	 */
	protected function _pushFileToArray(&$array, $file) {
		if (!is_array($file)) {
			array_push($array, $file);
		}
		else 
		{
			foreach ($file as $key => $value) {
				$array[$key] = $value;
			}
		}
		$this->_removeDuplicates($array);
		$this->_manifest_changed = TRUE;
	}

	/**
	 * pushes an files array to the given array
	 * @param array as reference
	 * @param array
	 */
	protected function _mergeFilestoArray(&$array, $files_array) {
		if (is_array($files_array)) {
			$array = array_merge($array, $files_array);
			$this->_removeDuplicates($array);
			$this->_manifest_changed = TRUE;
		}
	}

	/**
	 * returns a version hash encoded with sha1()
	 * @return string
	 */
	protected function _generateVersionHash()
	{
		$this->_version_hash = sha1(time()+microtime());
		return $this->_version_hash;
	}

	/**
	 * removes duplicates from an array
	 * @param array as reference
	 * @return void
	 */
	protected function _removeDuplicates(&$array)
	{
		$array_without_dups = array();
		foreach ($array as $key => $value) {
			if( ! in_array($value, $array_without_dups, TRUE)) {
				$array_without_dups[$key] = $value;
			}
		}
		
		$array = &$array_without_dups;
	}
	
	/**
	 * returns true if file and the contents of this object are equal
	 * @return boolean
	 */
	 function _fileChanged() {
		$file_contents = @file_get_contents($this->_manifest_path);
		
		// if file could not be opened
		if($file_contents === FALSE) {
			return TRUE;
		}
		// strip hash tag
		$file_contents = trim(preg_replace('/# [a-z0-9]+\n/', '', $file_contents));
		//$file_contents = explode("\n",$file_contents);
		$manifest_contents = trim($this->build(TRUE));
		
		return $file_contents !== $manifest_contents;
	}
	
	function addCacheFile($cache_file) {
		$this -> _pushFileToArray($this -> _cache_files, $cache_file);
	}

	function addNetworkFile($network_file) {
		$this -> _pushFileToArray($this -> _network_files, $network_file);
	}

	function addFallback($dynamic_file, $fallback) {
		$this -> _pushFileToArray($this -> _fallbacks, array($dynamic_file => $fallback));
	}

	function addCacheFiles($cache_files) {
		$this -> _mergeFilestoArray($this -> _cache_files, $cache_files);
	}

	function addNetworkFiles($network_files) {
		$this -> _mergeFilestoArray($this -> _network_files, $network_files);
	}

	function addFallbacks($fallbacks) {
		$this -> _mergeFilestoArray($this -> _fallbacks, $fallbacks);
	}

	function setCacheFiles($cache_files) {
		if (is_array($cache_files)) {
			$this -> _cache_files = $cache_files;
			$this->_manifest_changed = TRUE;
		}
	}

	function setNetworkFiles($network_files) {
		if (is_array($network_files)) {
			$this -> _network_files = $network_files;
			$this->_manifest_changed = TRUE;
		}
	}

	function setFallbacks($fallbacks) {
		if (is_array($fallbacks)) {
			$this -> _fallbacks = $fallbacks;
			$this->_manifest_changed = TRUE;
		}
	}

	function getCacheFiles() {
		return $this -> _cache_files;
	}

	function getNetworkFiles() {
		return $this -> _network_files;
	}

	function getFallbacks() {
		return $this -> _fallbacks;
	}

	function setManifestPath($file_path) {
		$this -> _manifest_path = $file_path;
	}

	function getManifestPath() {
		return $this -> _manifest_path;
	}
	
	function refresh($only_if_file_changed = FALSE) {
		$write_succeed = FALSE;
		
		if($this->getManifestPath() === '') {
			throw new Exception('You have to declare a Manifest filename first! Use setManifestPath($f) for this step.');
			exit(-1);
		} 
		
		if( $this->_fileChanged() && $this->_manifest_changed )
		{
			$write_succeed = $this->build();
		}
		else 
		{
			if (! $only_if_file_changed)
			{
				$append_value = "# " . $this->_generateVersionHash() . "\n";
				$write_succeed = file_put_contents($this->getManifestPath(), $append_value, FILE_APPEND);
			}
		}
		
		return $write_succeed !== FALSE;
	}

	function build($return_results = FALSE) {
		if($this->getManifestPath() === '') {
			throw new Exception('You have to declare a Manifest filename first! Use setManifestPath($f) for this step.');
			exit(-1);
		} 
		
		$write_succeed = FALSE;
		$write_value = "CACHE MANIFEST\n";
		foreach ($this->getCacheFiles() as $file) {
			$write_value .= "$file\n";
		}
		$write_value .= "NETWORK:\n";
		foreach ($this->getNetworkFiles() as $file) {
			$write_value .= "$file\n";
		}
		$write_value .= "FALLBACK:\n";
		foreach ($this->getFallbacks() as $accessed_file => $fallback) {
			$write_value .= "$accessed_file $fallback\n";
		}
		
		if($return_results === TRUE)
		{
			// return value without hash 
			return $write_value;
		}
		else 
		{
			$write_value .= "# ".$this->_generateVersionHash()."\n";
			$write_succeed = file_put_contents($this->getManifestPath(), $write_value);
		}
		
		$this->_manifest_changed = FALSE;
		
		return $write_succeed !== FALSE;
	}
}

/**
 * fast unit testing - wrap as a comment to disable
 
function debug($val, $name='')
{
	echo "<h4>".($name!==''?$name:'debug()')."</h4><pre>";
	var_dump($val);
	echo "</pre>";
}

function test($value, $expected_value, $name='')
{
	print($value === $expected_value?'success':'failed');
	print("\n");
}

$b = new Manifest_Builder();
// empty tests
test($b->getManifestPath(), '');
$b->setManifestPath('cache.manifest');
test($b->getManifestPath(), 'cache.manifest');

test($b->getCacheFiles(), array());

// check one file test
$b->addCacheFile('test_file.php');
test($b->getCacheFiles(), array('test_file.php'));
$b->addFallback('/', '/offline.html');
test($b->getFallbacks(), array('/'=>'/offline.html'));
$b->addNetworkFile('test_file.php');
test($b->getNetworkFiles(), array('test_file.php'));

// check adding mutiple files
$b->addCacheFile('test_file2.php');
$b->addCacheFile('test_file3.php');
$b->addCacheFiles(array('test_file4.php','test_file5.php'));
test($b->getCacheFiles(), array('test_file.php','test_file2.php','test_file3.php','test_file4.php','test_file5.php'));

$b->addFallback('/2', '/offline2.html');
$b->addFallback('/3', '/offline3.html');
$b->addFallbacks(array('/4'=>'/offline4.html'));
test($b->getFallbacks(), array('/'=>'/offline.html','/2'=>'/offline2.html','/3'=>'/offline3.html','/4'=>'/offline4.html'));

$b->addNetworkFile('test_file2.php');
$b->addNetworkFile('test_file3.php');
$b->addNetworkFiles(array('test_file4.php','test_file5.php'));
test($b->getNetworkFiles(), array('test_file.php','test_file2.php','test_file3.php','test_file4.php','test_file5.php'));

// check setting something
$b->setCacheFiles(array());
$b->setFallbacks(array());
$b->setNetworkFiles(array());
test($b->getCacheFiles(), array());
test($b->getNetworkFiles(), array());
test($b->getFallbacks(), array());

test($b->_fileChanged(), FALSE);
$b->refresh();
/**/